"use strict";

this.name = "Elite Trader";
this.author = "Reval";
this.description = "A profitable Market trade scores toward Elite";
this.version = "1.4";

/*  Version 1.4
	
	This OXP takes no account of cargo contracts,
	special deals, scoops of drifting cargo in space, 
	or of gems or precious metals.
	
*/

	
this.playerSoldCargo = function(commodity, units, price) {

	// selling preliminaries - update quantity and price
	if (this.$etLastPrice[commodity]>0) {
		this.$etSoldUnits += units;
		this.$etSoldPrice += price * units;
	}
	
	// see if we made a profit
	this.$etMadeProfit = ((this.$etSoldPrice > this.$etBoughtPrice) && (this.$etLastPrice[commodity] > 0));
	
	// conditions for scoring
	this.$etOKtoScore = ((units > 2) && (!this.$etDone) && (this.$etBoughtUnits>0) && (this.$etMadeProfit) && (!this.$etIsBellig) && (!this.$etIsScooper));
	
	// decrement the manifest by # of units and price
	if (this.$etLastPrice[commodity]>0) {
		this.$etBoughtUnits -= units;
		this.$etBoughtPrice -= this.$etLastPrice[commodity] * units;
	}
	
	// display ship's cargo and its purchase value
	if (price==this.$etLastPrice[commodity])
		player.consoleMessage("In hold: "+this.$etBoughtUnits+" pods ( "+formatCredits(((this.$etBoughtPrice)/10),true,true)+" )");

	// credit a "market killing" on first profitable sell
	if (this.$etOKtoScore) {
		player.score ++;
		this.$etPoints ++;
		this.$etHasScored = true;
		player.consoleMessage("You made a killing!");
		// for simplicity, only one score per station
		this.$etDone = true;
	// otherwise acknowledge a skillful trade if we're not just unloading
	} else if (this.$etMadeProfit && (price!=this.$etLastPrice[commodity]) && (this.$etStation!=this.$etLastStation)) 
		player.consoleMessage("Nice trade!");
}
	

this.playerBoughtCargo = function (commodity, units, price) {
	// buying preliminaries - record quantity and price
	this.$etBoughtUnits += units;
	this.$etBoughtPrice += price * units;
	this.$etLastPrice[commodity] = price;
	// display ship's cargo and its purchase value
	player.consoleMessage("In hold: "+this.$etBoughtUnits+" pods ( "+formatCredits(((this.$etBoughtPrice)/10),true,true)+" )");
}


this.guiScreenChanged = function(to, from) {
	if (to == "GUI_SCREEN_MARKET") {
		// suggest a G.E.T. presence on the market floor
		player.consoleMessage("The "+this.$etSystem+" Guild of Elite Traders", 5);
		// show hold-status on entering Market
		var pods = this._etPodCount();
		player.consoleMessage("In hold: "+pods+" pods ( "+formatCredits(((this.$etBoughtPrice)/10),true,true)+" )");
		// advise player to unload and re-buy cargo after game-load
		if ((pods>0) && (this.$etBoughtPrice<=0)) player.consoleMessage("Elite Trader: Please sell cargo and re-buy.",5); 
	} else
	if (to == "GUI_SCREEN_MANIFEST") {
		// suggest the G.E.T. presence on the Manifest screen
		player.consoleMessage("Under the aegis of the "+this.$etSystem+" G.E.T", 10);
	} else 
	if (to == "GUI_SCREEN_STATUS")
		if (this.$etIsWelcomed) {
			if (!this.$etMadeProfit)
				player.consoleMessage("We trust your stay at "+this.$etSystem+" proves profitable!", 10);
			else player.consoleMessage(this.$etSystem+" is your lucky star!", 10);
		} else {
			player.consoleMessage("The "+this.$etSystem+" Guild of Elite Traders welcomes you!", 5);
			this.$etIsWelcomed = true;
		}
}


this.shipDockedWithStation = function(station) {
	this.$etLastStation = this.$etStation;
	this.$etStation = station;
	this.$etLastSystem = this.$etSystem;
	// System's G.E.T. welcomes the commander on arrival
	player.consoleMessage("The "+this.$etSystem+" Guild of Elite Traders welcomes you!", 5);
	// clear flags and sold-quantities on docking
	this.$etDone = false;
	this.$etMadeProfit = false;
	this.$etHasScored = false;
	this.$etSoldUnits = 0;
	this.$etSoldPrice = 0;
	// leave an identical ship at the shipyard
	// in case player wishes to reset its specs
	this._etCloneShip();
}


this.shipWillDockWithStation = function(station) {
	// refuel a good brother's ship gratis
	var f ="";
	if ((player.ship.fuel<7) && (!this.$etIsBellig)) {
		player.ship.fuel = 7;
		f = "refuelled and "; }
	// if the Brother's recent trades were satisfactory...
	if (this.$etHasScored) {
		// a personal welcome from the local Guild Master
		player.addMessageToArrivalReport("The "+this.$etSystem+" Guild of Elite Traders bids their Brother "+player.name+" a hearty welcome to their Guild Home and trusts this sojourn among us will be advantageous...");
		player.addMessageToArrivalReport("Trade well! May the GETTING exceed the Taking!");
		// service the Brother's ship if its condition is under par
		if ((player.ship.serviceLevel<100) || (f!="")) {
			player.ship.serviceLevel = 100;
			player.addMessageToArrivalReport("Our Brother mechanic now reports that your craft is "+f+"in good repair. Fair voyage when you leave us, and return soon!");
		}
		// incremental protective enhancements for the Brother's ship
		if (this.$etLastSpeed<this.$etLimitSpeed) {
			player.ship.maxSpeed++;
			// check
			var spd = player.ship.maxSpeed;
		}
		if (this.$etLastThrust<this.$etLimitThrust) {
			player.ship.maxThrust++;
		}
		if (this.$etLastShields<this.$etLimitShields) {
			player.ship.maxForwardShield++;
			player.ship.maxAftShield++;
		}
		if (this.$etLastEnergy<this.$etLimitEnergy) {
			player.ship.maxEnergy++;
		}
		if (this.$etLastRecharge<this.$etLimitRecharge) {
			player.ship.forwardShieldRechargeRate++;
			player.ship.aftShieldRechargeRate++;
		}
		player.addMessageToArrivalReport("Our G.E.T. Brother adept has used an ages-lost and arcane technique to increase the capacity of your shields and your ship's speed by a small factor, that your passages in and out of our system might be a little safer.");
		if (spd!=this.$etLastSpeed) player.addMessageToArrivalReport("Previous speed: "+this.$etLastSpeed+";  Revised speed: "+player.ship.maxSpeed);
		player.addMessageToArrivalReport("Previous energy: "+this.$etLastEnergy+";  Revised energy: "+player.ship.maxEnergy);
		player.addMessageToArrivalReport("Previous recharge: "+this.$etLastRecharge+";  Revised recharge: "+player.ship.forwardShieldRechargeRate);
		// update last specs after report
		this.$etLastSpeed=player.ship.maxSpeed;
		this.$etLastThrust=player.ship.maxThrust;
		this.$etLastShields=player.ship.maxForwardShield;
		this.$etLastEnergy=player.ship.maxEnergy;
		this.$etLastRecharge=player.ship.forwardShieldRechargeRate;
	}
}


this.shipWillLaunchFromStation = function(station) {
	// G.E.T. ensures the good Brother's record is always clean
	if (!this.$etIsBellig) player.ship.bounty = 0;
}


this.shipLaunchedFromStation = function(stationLaunchedFrom) {
	player.commsMessage("The "+this.$etSystem+" G.E.T. Aegis wishes you clear lanes and an uneventful passage. Return safe!", 6);
}


this.playerStartedJumpCountdown = function(type, seconds) {
	player.commsMessage("Be careful out among the miscreants Brother!",6);
	if ((!this.$etIsBellig) && (!this.$etIsScooper)) player.ship.massLockable = false;
}


this.shipEnteredStationAegis = function(station) {
	player.commsMessage("The Galactic Guild of Elite Traders welcomes you to "+this.$etSystem+" Aegis! We are at your disposal in the shipyard. Trade well!", 6);
	player.ship.massLockable = true; // no slamming into station
}


this.shipKilledOther = function(whom, damageType) {
     // G.E.T. disapproval
	player.commsMessage(this.$etSystem+" G.E.T. notes their Brother's lapse...");
	this.$etKillCount++;
	this.$etIsBellig = (this.$etKillCount>0); // temporary
	this.$etBeenBellig = true; // permanent - a blot on the record
}


this.shipScoopedOther = function(whom) {
     // G.E.T. disapproval
	player.commsMessage(this.$etSystem+" G.E.T. notes their Brother's lapse...");
	this.$etIsScooper = true;	
}


this.shipExitedWitchspace = function() {
	this.$etSystem = System.systemNameForID(system.ID);
	this.$etIsWelcomed = false;
	if (this.$etKillCount>0) this.$etKillCount--;
	this.$etIsBellig = (this.$etKillCount>0);
	this.$etIsScooper = false;
}


this.playerBoughtNewShip = function(ship, price) {
	// spec globals to ship's initial values
	this._etResetSpecs();
	this.$etShipChanged = true;
}


this.playerWillSaveGame = function(message) {
       missionVariables.etSpeed = this.$etLastSpeed;
       missionVariables.etThrust = this.$etLastThrust;
       missionVariables.etEnergy = this.$etLastEnergy;
       missionVariables.etShields = this.$etLastShields;
       missionVariables.etRecharge = this.$etLastRecharge;
	   missionVariables.etPoints = this.$etPoints;
	   missionVariables.etKills = this.$etKillCount;
}


this.startUpComplete = function () {
	this.$etSystem = System.systemNameForID(system.ID);
	player.consoleMessage("The "+this.$etSystem+" Guild of Elite Traders welcomes you!", 5);
	this._etResetSpecs();
	// Load saved globals, if present
	var spd = missionVariables.etSpeed;
	var thr = missionVariables.etThrust;
	var ene = missionVariables.etEnergy;
	var shi = missionVariables.etShields;
	var rec = missionVariables.etRecharge;
	
	if (spd>=this.$etInitialSpeed) this.$etLastSpeed = spd;
	if (thr>=this.$etInitialThrust) this.$etLastThrust = thr;
	if (ene>=this.$etInitialEnergy) this.$etLastEnergy = ene;
	if (shi>=this.$etInitialShields) this.$etLastShields = shi;
	if (rec>=this.$etInitialRecharge) this.$etLastRecharge = rec;
	
	if (spd>=this.$etInitialSpeed) player.ship.maxSpeed = spd;
	if (thr>=this.$etInitialThrust) player.ship.maxThrust = thr;
	if (ene>=this.$etInitialEnergy) player.ship.maxEnergy = ene;
	if (shi>=this.$etInitialShields) player.ship.maxForwardShield = shi;
	if (rec>=this.$etInitialRecharge) player.ship.forwardShieldRechargeRate = rec;
	
	// load points here too
	var pts = missionVariables.etPoints; if (pts>0) this.$etPoints = pts;
	var kil = missionVariables.etKills; if (kil>0) this.$etKillCount = kil; 
	// loading globals complete.
	if (this._etHasFE()) log(this.name, "finds FE Ships present ("+player.ship.shipClassName+")");
}


this.startUp = function() {
    log(this.name, "Initialising OXP " + this.name);
	log(this.name, "detects version " + oolite.versionString);
	// declare & initialize globals
	this.$etDone = false;
	this.$etMadeProfit = false;
	this.$etOKtoScore = false;
	this.$etHasScored = false;
	this.$etIsWelcomed = false;
	this.$etShipChanged = false;
	this.$etIsExcellent = false; // points>??
	this.$etIsScooper = false;
	this.$etIsBellig = false; // temporary
	this.$etBeenBellig = false; // permanent
	this.$etKillCount = 0; // ship-kills
	this.$etPoints = 0; // G.E.T. Elite-points scored
	this.$etSoldUnits = 0;
	this.$etSoldPrice = 0;
	this.$etBoughtPrice = 0;
	this.$etBoughtUnits = 0;
	this.$etSystem = "";
	this.$etLastSystem = "";
	this.$etNextSystem = "";
	this.$etStation=0;
	this.$etLastStation=0;
	this.$etLastPrice = new Array();
	for (var c=0; c<20; c++) this.$etLastPrice[c]=0;
	
	this.$etInitialSpeed = 0;
	this.$etInitialThrust = 0;
	this.$etInitialEnergy = 0;
	this.$etInitialShields = 0;
	this.$etInitialRecharge = 0;

	this.$etLastSpeed = 0;
	this.$etLastThrust = 0;
	this.$etLastEnergy = 0;
	this.$etLastShields = 0;
	this.$etLastRecharge = 0;
	
	this.$etLimitSpeed = 470;
	this.$etLimitThrust = 256;
	this.$etLimitEnergy = 512;
	this.$etLimitShields = 512;
	this.$etLimitRecharge = 512;
	
}


// # of cargo pods in ship's hold
this._etPodCount = function() {
	var result = 0;
	for (var i=0; i<manifest.list.length; i++) {
		var q = manifest.list[i].quantity;
		if (q > 0) result+= q;
	}	
	return result;
}


// is FE Ships installed?
this._etHasFE = function() {
	var str = player.ship.shipClassName;
	var n = str.search(" FE");
	return (n>-1);
}


// leave an identical ship at the shipyard
// in case player wishes to reset its specs
this._etCloneShip = function() {
	if (oolite.versionString>"1.88") {
		var pr = player.ship.price * 0.75;
		system.mainStation.addShipToShipyard({ 
			short_description: "G.E.T. offers you this clone of your ship, to reset our augmented specs to nominal, should you desire a clean break.", 
			shipdata_key: player.ship.dataKey, 
			price: pr, 
			personality:0,
			extras:[""]
		});
	}
}


this._etResetSpecs = function() {
	this.$etLastSpeed = player.ship.maxSpeed;
	this.$etLastThrust = player.ship.maxThrust;
	this.$etLastShields = player.ship.maxForwardShield;
	this.$etLastEnergy = player.ship.maxEnergy;
	this.$etLastRecharge = player.ship.forwardShieldRechargeRate;
	// note ship's initial specs (for later cmparisons)
	this.$etInitialSpeed = player.ship.maxSpeed;
	this.$etInitialThrust = player.ship.maxThrust;
	this.$etInitialShields = player.ship.maxForwardShield;
	this.$etInitialEnergy = player.ship.maxEnergy;
	this.$etInitialRecharge = player.ship.forwardShieldRechargeRate;
}
